# Generated by Django 2.2.5 on 2019-09-09 13:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0062_auto_20190908_0610'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages',
            options={'verbose_name': 'Сообщения'},
        ),
    ]
