# Generated by Django 2.1.7 on 2019-03-07 00:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0010_auto_20190306_1418'),
    ]

    operations = [
        migrations.AlterField(
            model_name='params',
            name='key',
            field=models.TextField(),
        ),
    ]
