# Generated by Django 2.2.3 on 2019-07-06 22:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0014_auto_20190706_2103'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Uuid_2_id',
        ),
    ]
