# Generated by Django 2.2.4 on 2019-08-30 22:50

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('twits', '0003_auto_20190822_2250'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Chat_user_permission',
            new_name='Chat_user_user',
        ),
        migrations.DeleteModel(
            name='Chat_permission',
        ),
    ]
