# Generated by Django 2.2.2 on 2019-06-10 15:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0006_auto_20190610_1533'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documents_param_1c',
            name='document',
        ),
    ]
