# Generated by Django 2.2.4 on 2019-08-21 16:40

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0016_remove_holidays_props'),
    ]

    operations = [
        migrations.AddField(
            model_name='holidays',
            name='day_type',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='clndr.Day_types'),
        ),
    ]
