# Generated by Django 2.2.1 on 2019-05-26 10:42

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0003_files_1c'),
    ]

    operations = [
        migrations.AlterField(
            model_name='files_1c',
            name='props',
            field=bitfield.models.BitField((('imp', 'Импорт'), ('exp', 'Экспорт')), db_index=True, default=None),
        ),
    ]
