# Generated by Django 2.2.4 on 2019-08-05 22:39

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0032_auto_20190805_2238'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='user',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]
