# Generated by Django 3.0.8 on 2020-07-13 05:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0054_auto_20200713_0552'),
    ]

    operations = [
        migrations.AlterField(
            model_name='params',
            name='value',
            field=models.TextField(),
        ),
    ]
