# Generated by Django 2.2.7 on 2019-11-16 13:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0009_auto_20191116_1312'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='ip_address',
            field=models.CharField(max_length=20),
        ),
        migrations.AlterField(
            model_name='history',
            name='user_agent',
            field=models.CharField(max_length=255),
        ),
    ]
