# Generated by Django 2.2.4 on 2019-08-24 12:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0046_auto_20190824_1257'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day',
            name='numdayinweek',
            field=models.PositiveIntegerField(unique=True),
        ),
    ]
