# Generated by Django 2.2.4 on 2019-08-28 20:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0067_auto_20190826_0659'),
    ]

    operations = [
        migrations.AddField(
            model_name='day_types',
            name='length',
            field=models.TimeField(blank=True, null=True),
        ),
    ]
