# Generated by Django 2.2.4 on 2019-08-05 21:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0029_message_user_2_user'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Message_user_2_user',
        ),
    ]
