# Generated by Django 2.2.4 on 2019-08-22 22:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0026_auto_20190822_2217'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day',
            name='begindinnertime',
            field=models.TimeField(db_index=True, default=None, verbose_name='Начало перерыва на обед'),
        ),
        migrations.AlterField(
            model_name='shift_day',
            name='enddinnertime',
            field=models.TimeField(db_index=True, default=None, verbose_name='Конец перерыва на обед'),
        ),
    ]
