# Generated by Django 3.1 on 2020-08-08 04:28

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('history', '0013_auto_20200208_1133'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='ip_address',
            field=models.GenericIPAddressField(),
        ),
    ]
