# Generated by Django 2.2.2 on 2019-06-11 10:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0026_auto_20190610_1436'),
    ]

    operations = [
        migrations.DeleteModel(
            name='History',
        ),
    ]
