# Generated by Django 2.2.6 on 2019-10-08 17:14

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0031_user_props'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
    ]
