# Generated by Django 2.2.3 on 2019-07-06 11:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0011_auto_20190706_1040'),
    ]

    operations = [
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_boolean',
            field=models.BooleanField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_float',
            field=models.FloatField(blank=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_int',
            field=models.IntegerField(blank=True, db_index=True, null=True),
        ),
    ]
